uniform sampler2D texture;
uniform sampler2D bumpMap;
uniform vec3 lightDiffuse;

varying vec4 vertPos;
varying vec4 vertColor;
varying vec3 vertNormal;
varying vec3 vertLightDir;
varying vec4 vertTexCoord;
varying vec4 vertPointToEye;

vec2 dHdxy_fwd(vec2 vUv) {
  vec2 dSTdx = dFdx(vUv);
  vec2 dSTdy = dFdy(vUv);

  float bumpScale = 0.5;
  float Hll = bumpScale * texture2D(bumpMap, vUv).x;
  float dBx = bumpScale * texture2D(bumpMap, vUv + dSTdx).x - Hll;
  float dBy = bumpScale * texture2D(bumpMap, vUv + dSTdy).x - Hll;

  return vec2(dBx, dBy);
}

vec3 perturbNormalArb(vec3 surf_pos, vec3 surf_norm, vec2 dHdxy) {
  vec3 vSigmaX = dFdx(surf_pos);
  vec3 vSigmaY = dFdy(surf_pos);
  vec3 vN = surf_norm;		// normalized
  
  vec3 R1 = cross(vSigmaY, vN);
  vec3 R2 = cross(vN, vSigmaX);

  float fDet = dot(vSigmaX, R1);

  vec3 vGrad = sign(fDet) * (dHdxy.x * R1 + dHdxy.y * R2);
  return normalize(abs(fDet) * surf_norm - vGrad);
}

void main() {
  vec4 texColor = texture2D(texture, vertTexCoord.st);	


  vec3 bumpNorm = perturbNormalArb(normalize(vertPos.xyz), vertNormal, dHdxy_fwd(vertTexCoord.st));

  float lumi = abs(dot(vertLightDir, bumpNorm));
  float refl = dot(vertPointToEye.xyz, bumpNorm);

  vec4 diffuse = vec4(lumi, lumi, lumi, 1) * vec4(lightDiffuse, 1);
  vec4 specular = pow(max(0.0, refl), 10) * vec4(0, 1, 1, 1);

  gl_FragColor = (diffuse + specular) * texColor * vertColor;  
}